/*
 * BarPlot.java
 *
 * Created on 20 de marzo de 2007, 11:45
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.coti.graphics;

import com.coti.tools.JFrameAI;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Stroke;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.text.NumberFormat;
import java.util.Locale;
import javax.swing.JPanel;

/**
 *
 * @author bruegel
 */
public class BarPlot extends JPanel {
    private boolean preserveAspectRatio;
    private double[][] data;
    private double lx, ly, hx, vy;
    private double fx, fy, dx, dy;
    private Component owner;
    private double pXmin, pXmax, pYmin, pYmax;
    private double xmin;
    private double barWidth;
    private double xmax, ymin, ymax, temp;
    private Font titleFont, numberFont;
    private String plotTitle,xAxeLegend,yAxeLegend;
    private FontMetrics titleFM, numberFM;
    private boolean horizontalGridlines, verticalGridlines;
    
    public BarPlot(Component o, double[][] d, boolean preserveAspectRatio) {
        super();
        
        this.data = d;
        this.preserveAspectRatio = preserveAspectRatio;
        this.owner = o;
        
        this.numberFont = new Font("monospaced",Font.PLAIN,14);
        this.titleFont = new Font("serif",Font.BOLD,18);
        this.titleFM = this.getFontMetrics(titleFont);
        this.numberFM = this.getFontMetrics(numberFont);
    }
    
    public BarPlot(Component o, boolean preserveAspectRatio) {
        super();
        
        data = null;
        this.preserveAspectRatio = preserveAspectRatio;
        this.owner = o;
        
        this.numberFont = new Font("monospaced",Font.PLAIN,14);
        this.titleFont = new Font("serif",Font.BOLD,18);
        this.titleFM = this.getFontMetrics(titleFont);
        this.numberFM = this.getFontMetrics(numberFont);
    }
    
    public BarPlot(double[][] d, String[] title_and_units){
        super();
        
        this.data = d;
        this.preserveAspectRatio = false;
        this.owner = null;
        this.plotTitle = title_and_units[0];
        this.xAxeLegend = title_and_units[1];
        this.yAxeLegend = title_and_units[2];
        
        this.numberFont = new Font("monospaced",Font.PLAIN,14);
        this.titleFont = new Font("serif",Font.BOLD,18);
        this.titleFM = this.getFontMetrics(titleFont);
        this.numberFM = this.getFontMetrics(numberFont);
        
    }
    public void calculateTransformFactors() {
       /*
        * Pick up the size of this component; if too small
        * a 300x200 rectangle will be used.
        *
        * Also, it defines the size and placement of the
        * destination rectangle
        * lx - the width
        * ly - the height
        * hx - distance from the left border of the panel
        * vy - distance from the top border of the panel
        *
        * These are final measurements.
        */
        int panelWidth, panelHeight;
        
        panelWidth = this.getSize().width;
        panelHeight = this.getSize().height;
        
        lx = panelWidth  *7/10;
        ly = panelHeight *7/10;
        hx = panelWidth  *2/10;
        vy = panelHeight *1/10;
        /*
         * This method produces the following parameters:
         *
         * double fx - X-axe scaling
         * double fy - Y-axe scaling
         * double dx - X-axe displacement
         * double dy - Y-axe displacement
         *
         * The actual size of the primitive rectangle is increased by
         * 10% on all sides when calculating these factors. Thus the
         * real points will never reach the borders of the destination
         * rectangle.
         *
         * This transform intends to proyect the expanded primitive
         * rectangle onto the destination rectangle.
         */
        if (null==data) {
            JFrameAI.errMess(owner,"No data","BarPlot Error");
        }
        
        xmin = xmax = data[0][0];
        ymin = ymax = data[0][1];
        /*
         * Get the primitive rectangle
         */
        for(int i=0;i<data.length;i++) {
            temp = data[i][0];
            if (temp < xmin) xmin = temp; else if (temp>xmax) xmax = temp;
            temp = data[i][1];
            if (temp < ymin) ymin = temp; else if (temp>ymax) ymax = temp;
        }
        /*
         * Build the expanded primitive rectangle
         */
        
        pXmin = xmin - 0.1*(xmax-xmin);
        pXmax = xmax + 0.1*(xmax-xmin);
        pYmin = ymin - 0.1*(ymax-ymin);
        pYmax = ymax + 0.1*(ymax-ymin);
        
        fx = lx/(pXmax-pXmin);
        fy = ly/(pYmax-pYmin);
        /*
         * On occasion, we will want a plot in which squares
         * turn into squares. This is how to do it. Normally the
         * x dimension will be bigger, but this code makes sure
         * of getting the correct scaling.
         */
        if (preserveAspectRatio) {
            fx = Math.min(fx,fy);
            fy = fx;
        }
        dx = -pXmin*fx;
        dy = -pYmin*fy;
        barWidth = (xmax-xmin)/(2.0*data.length);
    }
    
    public Point2D.Double transform(Point2D.Double aPoint) {
        Point2D.Double transformedPoint = new Point2D.Double();
        transformedPoint.x = hx + aPoint.x*fx+dx;
        transformedPoint.y = vy + ly -(aPoint.y*fy+dy);
        System.out.println(transformedPoint.x + " " + transformedPoint.y);
        return transformedPoint;
    }
    public Point2D.Double transform(double x, double y) {
        Point2D.Double transformedPoint = new Point2D.Double( hx + x*fx+dx,vy + ly -(y*fy+dy));
        System.out.println(transformedPoint.x + " " + transformedPoint.y);
        return  transformedPoint;
    }
    /*
     * (0,0) is the upper left corner
     */
    public void plotSegment(Graphics2D g,Point2D.Double start, Point2D.Double end) {
        g.draw(new Line2D.Double(start,end));
    }
    
    public boolean allValuesPositive() {
        for(int i=0;i<data.length;i++)
            if (data[i][1] <= 0)
                return false;
        return true;
    }
    
    public boolean allValuesPositiveOrZero() {
        for(int i=0;i<data.length;i++)
            if (data[i][1] < 0)
                return false;
        return true;
    }
    
    public boolean allValuesNegative() {
        for(int i=0;i<data.length;i++)
            if (data[i][1] >= 0)
                return false;
        return true;
    }
    
    public boolean allValuesNegativeOrZero() {
        for(int i=0;i<data.length;i++)
            if (data[i][1] > 0)
                return false;
        return true;
    }
    
    public boolean positiveAndNegativeValues(){
        boolean foundPositive = false, foundNegative = false;
        for(int i=0;i<data.length;i++){
            if (data[i][1] <0) foundNegative = true;
            if (data[i][1] >0) foundPositive = true;
        }
        return foundPositive && foundNegative;
    }
    
    public void plotBarFromPositiveYTopYmin(Graphics2D g, Point2D.Double upperLeft) {
        /*
         * The upperLeft argument indicates the upper edge of the bar,
         * which must come down to the X Axe. This axe is taken to be
         * at pYmin, since 0 might be a long way off and produce bar charts
         * that are excessively long in the tooth. It is a good compromise.
         *
         * This method should be used when all Y values are positive.
         */
        Point2D.Double lowerRight = new Point2D.Double();
        lowerRight.x = upperLeft.x+barWidth;
        lowerRight.y = pYmin;
        Point2D.Double tu = transform(upperLeft);
        Point2D.Double tl = transform(lowerRight);
        Rectangle2D.Double rd = new Rectangle2D.Double(tu.x-(lx/(5.0*data.length)),tu.y,tl.x-tu.x,tl.y-tu.y);
        g.fill(rd);
    }
    
    public void plotBarFromPositiveYToZero(Graphics2D g, Point2D.Double upperLeft) {
        /*
         * The upperLeft argument indicates the upper edge of the bar,
         * which must come down to the X Axe. This axe is taken to be
         * at pYmin, since 0 might be a long way off and produce bar charts
         * that are excessively long in the tooth. It is a good compromise.
         *
         * This method should be used when all Y values are positive.
         */
        Point2D.Double lowerRight = new Point2D.Double();
        lowerRight.x = upperLeft.x+barWidth;
        lowerRight.y = 0.0;
        Point2D.Double tu = transform(upperLeft);
        Point2D.Double tl = transform(lowerRight);
        Rectangle2D.Double rd = new Rectangle2D.Double(tu.x-(lx/(5.0*data.length)),tu.y,tl.x-tu.x,tl.y-tu.y);
        g.fill(rd);
    }
    
    public void plotBarFromZeroToNegativeY(Graphics2D g, Point2D.Double lowerLeft) {
        /*
         * The lowerLeft argument indicates the position of the lower edge
         * of the bar, which starts on the X Axe.
         */
        Point2D.Double upperLeft = new Point2D.Double();
        upperLeft.x = lowerLeft.x;
        upperLeft.y = 0;
        Point2D.Double lowerRight = new Point2D.Double();
        lowerRight.x = lowerLeft.x+barWidth;
        lowerRight.y = lowerLeft.y;
        Point2D.Double tu = transform(upperLeft);
        Point2D.Double tl = transform(lowerRight);
        /*
         * Bars are displaced left half a bar width
         */
        Rectangle2D.Double rd = new Rectangle2D.Double(tu.x-(lx/(5.0*data.length)),tu.y+1,tl.x-tu.x,tl.y-tu.y-1);
        g.fill(rd);
    }
    
    public void plotBarFrompYmaxToNegativeY(Graphics2D g, Point2D.Double lowerLeft) {
        /*
         * The lowerLeft argument indicates the position of the lower edge
         * of the bar, which starts on the X Axe.
         */
        Point2D.Double upperLeft = new Point2D.Double();
        upperLeft.x = lowerLeft.x;
        upperLeft.y = pYmax;
        Point2D.Double lowerRight = new Point2D.Double();
        lowerRight.x = lowerLeft.x+barWidth;
        lowerRight.y = lowerLeft.y;
        Point2D.Double tu = transform(upperLeft);
        Point2D.Double tl = transform(lowerRight);
        /*
         * Bars are displaced left half a bar width
         */
        Rectangle2D.Double rd = new Rectangle2D.Double(tu.x-(lx/(5.0*data.length)),tu.y+1,tl.x-tu.x,tl.y-tu.y-1);
        g.fill(rd);
    }
    
    
    public void paintComponent(Graphics g){
        Stroke oldStroke;
        super.paintComponent(g);
        this.calculateTransformFactors();
        Graphics2D g2 = (Graphics2D)g;
        /*
         * Expanded rectangle
         */
        plotSegment(g2, transform(pXmax,pYmin), transform(pXmax,pYmax));
        plotSegment(g2, transform(pXmax,pYmax), transform(pXmin,pYmax));
        plotSegment(g2, transform(pXmin,pYmax), transform(pXmin,pYmin));
        plotSegment(g2, transform(pXmin,pYmin), transform(pXmax,pYmin));
        /*
         * Primitive boundary rectangle
         */
        plotSegment(g2, transform(xmin,ymin), transform(xmax,ymin));
        plotSegment(g2, transform(xmax,ymin), transform(xmax,ymax));
        plotSegment(g2, transform(xmax,ymax), transform(xmin,ymax));
        plotSegment(g2, transform(xmin,ymax), transform(xmin,ymin));
        /*
         * This code shows the X Axis, if visible
         */
        if (ymin*ymax<=0) {
            Point2D.Double xAxeLeft = new Point2D.Double(xmin,0.0);
            Point2D.Double xAxeRight = new Point2D.Double(xmax,0.0);
            xAxeLeft.x -= barWidth/2;
            xAxeRight.x += barWidth/2;
            g.setColor(Color.RED);
            oldStroke = g2.getStroke();
            g2.setStroke(new BasicStroke(2.0f));
            plotSegment(g2, transform(xAxeLeft),transform(xAxeRight));
            g.setColor(Color.BLACK);
            g2.setStroke(oldStroke);// Leave stroke as it was
        }
     /*
      * This code shows horizontal marks and values. Will overlap if too many
      * bar or if bar values too close. No solution but showing arbitrary
      * vertical values and lines.
      *
      * Also always remember we work on primitive coordinates, then transform
      * them to screen coordinates. Some tweaking may be done after the
      * transform is done, since we know we work (then I say) in pixels.
      */
        NumberFormat nf = NumberFormat.getNumberInstance(Locale.US);
        nf.setMaximumFractionDigits(2);
        nf.setMinimumFractionDigits(2);
        nf.setMinimumIntegerDigits(2);
        
        Point2D.Double p = new Point2D.Double(), q = null;
        /*
         * Numbers should be right-aligned on the left hand side of the
         * expanded primitive regtangle. Actually once they are I move
         * them left 10 pixels.
         */
        p.x = pXmin; // Position of the left border of the expanded rectangle
        String tickValue;
        setFont(numberFont);
        if (horizontalGridlines) {
            for(int i=0;i<data.length;i++) {
                p.y = ymin+i*(ymax-ymin)/(data.length-1);
                q = transform(p);
                tickValue = nf.format(ymin+i*(ymax-ymin)/(data.length-1));
                g2.drawString(tickValue,(float)q.x - numberFM.stringWidth(tickValue)-10 ,(float)q.y);
            }
        /*
         * This code shows horizontal grid lines
         */
            
            Point2D.Double left, right, tLeft, tRight;
            left = new Point2D.Double();
            right = new Point2D.Double();
            left.x = pXmin;
            right.x = pXmax;
            g2.setPaint(Color.GREEN);
            for(int i=0;i<data.length;i++) {
                left.y = ymin+i*(ymax-ymin)/(data.length-1);
                right.y = left.y;
                tLeft = transform(left);
                tRight = transform(right);
                plotSegment(g2,tLeft,tRight);
            }
            
            /*
             *  This code shows the Y axis legend
             */
            g2.setPaint(Color.BLACK);
            double xPos = hx/2 ;
            double yPos = vy +ly/2f + numberFM.stringWidth(this.yAxeLegend)/2;
            g2.rotate(-1.57);
            g2.drawString(this.yAxeLegend,-(float)yPos,(float)xPos);
            g2.rotate(1.57);
            /*
             *  This code shows the X axis legend
             */
            xPos = hx + lx/2 - numberFM.stringWidth(this.xAxeLegend)/2;
            yPos = vy +ly + 2*(numberFM.getMaxDescent() + 10) + 10 ;
            g2.drawString(this.xAxeLegend,(float)xPos, (float)yPos);
            
        }
    /*
     * This code shows the bars, depending on the set of data
     */
        double x,y;
        g2.setPaint(Color.GREEN);
        if (allValuesPositive()){
            for(int i=0;i<data.length;i++) {
                x = data[i][0];
                y = data[i][1];
                Point2D.Double ulp = new Point2D.Double(x,y);
                plotBarFromPositiveYTopYmin(g2, ulp);
            }
        } else if (allValuesNegative()){
            for(int i=0;i<data.length;i++) {
                x = data[i][0];
                y = data[i][1];
                Point2D.Double ulp = new Point2D.Double(x,y);
                plotBarFrompYmaxToNegativeY(g2, ulp);
            }
        } else if (allValuesPositiveOrZero()){
            for(int i=0;i<data.length;i++) {
                x = data[i][0];
                y = data[i][1];
                Point2D.Double ulp = new Point2D.Double(x,y);
                plotBarFromPositiveYTopYmin(g2, ulp);
            }
        } else if (allValuesNegativeOrZero()) {
            for(int i=0;i<data.length;i++) {
                x = data[i][0];
                y = data[i][1];
                Point2D.Double ulp = new Point2D.Double(x,y);
                plotBarFrompYmaxToNegativeY(g2, ulp);
            }
        } else if (positiveAndNegativeValues()){
            for(int i=0;i<data.length;i++) {
                x = data[i][0];
                y = data[i][1];
                Point2D.Double ulp = new Point2D.Double(x,y);
                if (y >0)
                    plotBarFromPositiveYToZero(g2, ulp);
                else if (y<0)
                    plotBarFromZeroToNegativeY(g2, ulp);
            }
            
        }
        g2.setPaint(Color.BLACK);
        /*
         * This code shows vertical grid lines and ticks
         */
        if (verticalGridlines) {
            Point2D.Double down,up,tDown,tUp;
            down = new Point2D.Double();
            up = new Point2D.Double();
            down.y = pYmin;
            up.y = pYmax;
            for(int i=0;i<data.length;i++) {
                // Ticks
                g2.setPaint(Color.BLACK);
                x = data[i][0];
                y = pYmin ;
                p = new Point2D.Double(x,y);
                q = transform(p);
                tickValue = nf.format(x);
                g2.drawString(tickValue,
                        (float)q.x - numberFM.stringWidth(tickValue)/2,
                        (float)q.y + numberFM.getMaxAscent() + 5);
                // Gridlines
                g2.setPaint(Color.GREEN);
                tDown = transform(down);
                tUp = transform(up);
                tDown.x = q.x;
                tUp.x = q.x;
                plotSegment(g2,tDown,tUp);
            }
        }
        
        /*
         * This code shows the title string
         */
        g2.setPaint(Color.BLACK);
        p.x = pXmin + (pXmax-pXmin)/2;
        p.y = pYmax;
        q = transform(p);
        g2.setFont(titleFont);
        g2.drawString(plotTitle, (float)q.x - titleFM.stringWidth(plotTitle)/2, (float)q.y-titleFM.getMaxDescent()-5);
    }
    
    public void setPlotTitle(String plotTitle) {
        this.plotTitle = plotTitle;
    }
    
    public void setXAxeLegend(String xAxeLegend) {
        this.xAxeLegend = xAxeLegend;
    }
    
    public void setYAxeLegend(String yAxeLegend) {
        this.yAxeLegend = yAxeLegend;
    }
    
    public void setHorizontalGridlines(boolean horizontalGridlines) {
        this.horizontalGridlines = horizontalGridlines;
    }
    
    public void setVerticalGridlines(boolean verticalGridlines) {
        this.verticalGridlines = verticalGridlines;
    }
    public void setData(double[][] d) {
        this.data = d;
        this.revalidate();
    }
    public void setData(float[][] d) {
        data = new double[d.length][d[0].length];
        for(int i=0;i<d.length;i++)
            for(int j=0;j<d[0].length;j++)
                data[i][j]=d[i][j];
        this.revalidate();
    }
}
