package com.coti.tools;
/**
 * @author coti
 * @version 1.7
 * @since 1.5
 */
import javax.swing.*;
import java.awt.event.*;
/*
	Attention! As of v. 1.7 method names have changed!
*/
public class AuxMenu {
	/**
	 * This is the most simple version of makeOpt(). It only takes
	 * the JMenuItem's name and a reference that points to its action listener.
	 * @param name	A String to be used as the JMenuItem's text
	 * @param action The descriptor of an ActionListener
	 * @return	a JMenuItem with the given characteristics
	 */
	public static  JMenuItem makeOpt(String name, ActionListener action){
		JMenuItem jmi = null;
		if (null == action)
		{
			JOptionPane.showMessageDialog(	null,
																			"JMenuItem: null action not valid.",
																			"Fatal error...exiting.",
																			JOptionPane.ERROR_MESSAGE);
			System.exit(1);
		}
		else
		{
			jmi = new JMenuItem(name);
			jmi.addActionListener(action);
		}
		return jmi;
	}
	/**
	 * This version of makeOpt() produces a JMenuItem with a name,
	 * an action listener an a value for its action command.
	 * @param name	A String to be used as the JMenuItem's text
	 * @param action The descriptor of an ActionListener
	 * @param command	The actual ActionCommand string, to be used as
	 * an indication of the source component in the actionPerformed method
	 * of an ActionListener
	 * @return	a JMenuItem with the given characteristics
	 */
	public static  JMenuItem makeOpt(String name, ActionListener action, String command){
		JMenuItem jmi = null;
		if (null == action || null == command)
		{
			JOptionPane.showMessageDialog(	null,
																			"JMenuItem: null action or command.",
																			"Fatal error...exiting.",
																			JOptionPane.ERROR_MESSAGE);
			System.exit(1);
		}
		else
		{
			jmi = new JMenuItem(name);
			jmi.setActionCommand(command);
			jmi.addActionListener(action);
		}
		return jmi;
	}
	/**
	 * The most powerful version of makeOpt(). This menu item will have a text,
	 * an action listener, and a keyboard equivalent.
	 * @param name	A String to be used as the JMenuItem's text
	 * @param action The descriptor of an ActionListener
	 * @param vKey	An expression of the form KeyEvent.VK_A, KeyEvent.VK_B,
	 * KeyEvent.VK_ENTER, etc.
	 * @param mask	An expression of the form InputEvent.SHIFT_MASK,
	 * InputEvent.CTRL_MASK, InputEvent.META_MASK, InputEvent.ALT_MASK
	 * @return	a JMenuItem with the given characteristics
	 */
	public static  JMenuItem makeOpt(String name, ActionListener action, int vKey, int mask){
		JMenuItem jmi = null;
		if (null == action)
		{
			JOptionPane.showMessageDialog(	null,
																			"JMenuItem: null action not valid.",
																			"Fatal error...exiting.",
																			JOptionPane.ERROR_MESSAGE);
			System.exit(1);
		}
		else
		{
			jmi = new JMenuItem(name);
			jmi.addActionListener(action);
			KeyStroke k = KeyStroke.getKeyStroke(vKey,mask);
			jmi.setAccelerator(k);
		}
		return jmi;
	}
	/**
	 * Since we make frequent use of the control key as a modifier, this
	 * method assumes that the mask is InputEvent.CTRL_MASK and we only
	 * have to pass the virtual key.
	 * @param name	A String to be used as the JMenuItem's text
	 * @param action The descriptor of an ActionListener
	 * @param vKey	An expression of the form KeyEvent.VK_A, KeyEvent.VK_B,
	 * KeyEvent.VK_ENTER, etc.
	 * @return	a JMenuItem with the given characteristics
	 */
	public static  JMenuItem ctrlOpt(String name, ActionListener action, int vKey){
		JMenuItem jmi = null;
		if (null == action)
		{
			JOptionPane.showMessageDialog(	null,
																			"JMenuItem: null action not valid.",
																			"Fatal error...exiting.",
																			JOptionPane.ERROR_MESSAGE);
			System.exit(1);
		}
		else
		{
			jmi = new JMenuItem(name);
			jmi.addActionListener(action);
			KeyStroke k = KeyStroke.getKeyStroke(vKey,InputEvent.CTRL_MASK);
			jmi.setAccelerator(k);
		}
		return jmi;
	}
	/**
	 * Since we make frequent use of the meta key as a modifier, this method
	 * assumes that the mask is InputEvent.META_MASK and we only have to pass
	 * the virtual key.
	 * @param name	A String to be used as the JMenuItem's text
	 * @param action The descriptor of an ActionListener
	 * @param vKey	An expression of the form KeyEvent.VK_A, KeyEvent.VK_B,
	 * KeyEvent.VK_ENTER, etc.
	 * @return	a JMenuItem with the given characteristics
	 */
	public static  JMenuItem metaOpt(String name, ActionListener action, int vKey){
		JMenuItem jmi = null;
		if (null == action)
		{
			JOptionPane.showMessageDialog(	null,
																			"JMenuItem: null action not valid.",
																			"Fatal error...exiting.",
																			JOptionPane.ERROR_MESSAGE);
			System.exit(1);
		}
		else
		{
			jmi = new JMenuItem(name);
			jmi.addActionListener(action);
			KeyStroke k = KeyStroke.getKeyStroke(vKey,InputEvent.META_MASK);
			jmi.setAccelerator(k);
		}
		return jmi;
	}
	/**
	 * This methods produces a JMenu the given name; the JMenuItems passed
	 * as a second argument will be added to the menu.
	 * @param nameOfMenu	A String to be used as the JMenu's text
	 * @param listOfJMenuItems	A JMenuItem[] with the options shown in this JMenu
	 * @return	A JMenu with the given name and contents
	 */
	public static JMenu makeMenu(String nameOfMenu, JMenuItem[] listOfJMenuItems){
		if (null == nameOfMenu || null == listOfJMenuItems)
		{
			JOptionPane.showMessageDialog(	null, 
																			"JMenu: Null name or list of items.",
																			"Fatal error...exiting.",
																			JOptionPane.ERROR_MESSAGE);
			System.exit(2);
		}
		JMenu jm = new JMenu(nameOfMenu);
		for(JMenuItem option : listOfJMenuItems)
			{
				jm.add(option);
			}
		return jm;
	}
	/**
	 * This method produces a JMenuBar that contains the given set of menus.
	 * @param listOfJMenus	The set of menus to be shown in this menubar
	 * @return	A JMenuBar with the given menus
	 */
	public static JMenuBar makeMenuBar(JMenu[] listOfJMenus){
		if (null == listOfJMenus)
		{
			JOptionPane.showMessageDialog(	null,
																			"JMenubar: Null list of menus.",
																			"Fatal error...exiting.",
																			JOptionPane.ERROR_MESSAGE);
			System.exit(3);
		}
		JMenuBar jmb = new JMenuBar();
		for(JMenu menu : listOfJMenus)
			{
				jmb.add(menu);
			}
		return jmb;
		
	}
}
